function aPoint(x, y)
{
    var p = new Array;
    p.x = x; p.y = y;
    
    return p;
}

function nicenum(x)
{
    var xsign, f, y, fexp, rx, sx;
    
    if (x == 0.0) {
        return(0.0);
    }

    xsign = x/Math.abs(x);
    x = Math.abs(x);

    fexp = Math.floor(Math.log(x)/Math.log(10));
    sx = x/Math.pow(10.0, fexp)/10.0;       // scaled x
    rx = Math.floor(sx);                    // rounded x
    f = 10*(sx - rx);                       // fraction between 0 and 10

    if (f < 1.5) {
        y = 1;
    } else
    if (f < 3.0) {
        y = 2;
    } else
    if (f < 7.0) {
        y = 5;
    } else {
        y = 10;
    }
    
    sx = rx + y/10.0;
    
    return (xsign*sx*10.0*Math.pow(10.0, fexp));
}

qx.Class.define("pf.ui.canvas.Plotter",
{
    extend: pf.ui.canvas.Canvas,
    
    construct: function() {
        this.base(arguments);
        this.setCursor("crosshair");
        
        var v = new Array;
        v.xmin = 0.1;
        v.xmax = 0.9;
        v.ymin = 0.1;
        v.ymax = 0.9;
        var w = new Array;
        w.xmin = 0.0;
        w.xmax = 1.0;
        w.ymin = 0.0;
        w.ymax = 1.0;
        
        this.v = v;
        this.w = w;

        var tracker = new qx.legacy.ui.basic.Atom("X:<br>Y:");
        tracker.setBorder("groove");
        tracker.hide();
        tracker.dp = aPoint();
        tracker.vp = aPoint();
        tracker.wp = aPoint();
        this.add(tracker);
        this.tracker = tracker;
        
        this.setHorizontalChildrenAlign("left");
        this.setVerticalChildrenAlign("top");
        
        this.addListener("mousemove", function(ev) {
                var tracker = this.tracker;
                var el = this.getElement();
                tracker.dp.x = ev.getPageX() -
                    qx.legacy.html.Location.getClientInnerLeft(el) - 1;
                tracker.dp.y = ev.getPageY() -
                    qx.legacy.html.Location.getClientInnerTop(el) - 1;
                this.dPoint2vPoint(tracker.dp, tracker.vp);
                this.vPoint2wPoint(tracker.vp, tracker.wp);
                tracker.setLabel("X: " + pf.util.Format.numPrint(tracker.wp.x) +
                    "<br>Y: " + pf.util.Format.numPrint(tracker.wp.y));
                tracker.setLeft(tracker.dp.x + 12);
                tracker.setTop(tracker.dp.y - 12);
                
                if (this.isInsideViewport(tracker.vp)) {
                    tracker.show();
                } else {
                    tracker.hide();
                }
            }, this);
        
        this.addListener("mouseover", function(ev) {
                this.tracker.show();
            }, this);
        
        this.addListener("mouseout", function(ev) {
                this.tracker.hide();
            }, this);
    },
    
    members:
    {
        vPoint2dPoint: function(vp, p)
        {
            var width  = this.getWidthValue();
            var height = this.getHeightValue();
            
            p.x = Math.round(width*vp.x);
            p.y = Math.round(height*(1.0 - vp.y));
        },
        
        dPoint2vPoint: function(dp, vp)
        {
            var width  = this.getWidthValue();
            var height = this.getHeightValue();
            
            vp.x = dp.x/width;
            vp.y = 1.0 - dp.y/height;
        },
        
        wPoint2vPoint: function(wp, vp)
        {
            var v = this.v;
            var w = this.w;
            
            vp.x = v.xmin + (wp.x - w.xmin)/(w.xmax - w.xmin)*(v.xmax - v.xmin);
            vp.y = v.ymin + (wp.y - w.ymin)/(w.ymax - w.ymin)*(v.ymax - v.ymin);
        },
        
        vPoint2wPoint: function(vp, wp)
        {
            var v = this.v;
            var w = this.w;
            
            wp.x = w.xmin + (vp.x - v.xmin)/(v.xmax - v.xmin)*(w.xmax - w.xmin);
            wp.y = w.ymin + (vp.y - v.ymin)/(v.ymax - v.ymin)*(w.ymax - w.ymin);
        },
        
        drawLineV: function(vp1, vp2)
        {
            var jg = this.jg;
            if (!jg) {
                return;
            }
            
            p1 = aPoint();
            p2 = aPoint();
            
            this.vPoint2dPoint(vp1, p1);
            this.vPoint2dPoint(vp2, p2);
            
            jg.drawLine(p1.x, p1.y, p2.x, p2.y);
        },
        
        drawRectV: function(vp1, vp2)
        {
            var jg = this.jg;
            if (!jg) {
                return;
            }
            
            p1 = aPoint();
            p2 = aPoint();
            
            this.vPoint2dPoint(vp1, p1);
            this.vPoint2dPoint(vp2, p2);
            
            var x, y, width, height;
            if (p1.x < p2.x) {
                x = p1.x;
                width = p2.x - p1.x;
            } else {
                x = p2.x;
                width = p1.x - p2.x;
            }
            if (p1.y < p2.y) {
                y = p1.y;
                height = p2.y - p1.y;
            } else {
                y = p2.y;
                height = p1.y - p2.y;
            }
            
            jg.drawRect(x, y, width, height);
        },
        
        drawStringV: function(vp, str, halign, valign)
        {
            var jg = this.jg;
            if (!jg) {
                return;
            }
            
            p = aPoint();
            
            this.vPoint2dPoint(vp, p);
            
            this.drawString(p.x, p.y, str, halign, valign);
        },
        
        setViewportCoordinates: function(vxmin, vymin, vxmax, vymax)
        {
            this.v.xmin = vxmin;
            this.v.ymin = vymin;
            this.v.xmax = vxmax;
            this.v.ymax = vymax;
        },
        
        setWorldCoordinates: function(wxmin, wymin, wxmax, wymax)
        {
            this.w.xmin = Math.min(wxmin, wxmax);
            this.w.ymin = Math.min(wymin, wymax);
            this.w.xmax = Math.max(wxmin, wxmax);
            this.w.ymax = Math.max(wymin, wymax);
        },
        
        isInsideViewport: function(vp) {
            var v = this.v;
            if (vp.x >= v.xmin && vp.x <= v.xmax &&
                vp.y >= v.ymin && vp.y <= v.ymax) {
                return true;
            } else {
                return false;
            }
        },
        
        drawPolyLineW: function(x, y)
        {
            var jg = this.jg;
            if (!jg) {
                return;
            }
            
            var  p = aPoint();
            var vp = aPoint();
            var wp = aPoint();

            dx = new Array;
            dy = new Array;
            for (var i = 0; i < x.length; i++) {
                wp.x = x[i];
                wp.y = y[i];
                this.wPoint2vPoint(wp, vp);
                this.vPoint2dPoint(vp, p);
                dx.push(p.x);
                dy.push(p.y);
            }
            
            jg.drawPolyLine(dx, dy);
        },
        
        drawFrame: function(color)
        {
            var v = this.v;

            this.setColor(color);
            
            var vp1 = aPoint(v.xmin, v.ymin);
            var vp2 = aPoint(v.xmax, v.ymax);

            this.drawRectV(vp1, vp2);
        },

        drawLabel: function(axis, label, color)
        {
            var v = this.v;

            this.setColor(color);
            
            var vp;
            var offset = 0.04;
            var halign, valign;
            if (axis == "x") {
                vp = aPoint((v.xmin + v.xmax)/2, v.ymin - offset);
                
                halign = "center";
                valign = "top";
            } else {
                vp = aPoint(v.xmin - offset, (v.ymin + v.ymax)/2);
                
                halign = "right";
                valign = "middle";
            }
            
            this.drawStringV(vp, label, halign, valign);
        },
        
        drawTicks: function(axis, color)
        {
            var v = this.v;
            var w = this.w;
            
            // Number for autoticking
            var naticks = 6;

            this.setColor(color);
            
            var wp = aPoint();
            var vp1 = aPoint();
            var vp2 = aPoint();
            
            
            var halign, valign;
            if (axis == "x") {
                wmin  = w.xmin;
                wmax  = w.xmax;
                
                halign = "center";
                valign = "top";
            } else {
                wmin  = w.ymin;
                wmax  = w.ymax;
                
                halign = "right";
                valign = "middle";
            }
            var range = wmax - wmin;
            
            var wval;
            var tmajor = nicenum(range/(naticks - 1));
            
            var prec = Math.round(Math.log(Math.max(Math.abs(wmax),
                Math.abs(wmin))/tmajor)/Math.log(10));
            
            var nmajor = Math.ceil(range/tmajor + 1);
            var nticks = (nmajor - 1)*2 + 1;
            
            wstart = Math.floor(wmin/tmajor)*tmajor;
            
            for (var i = 0; i < nticks; i++) {
                wval = wstart + i*tmajor/2;
                if (axis == "x") {
                    wp.x = wval;
                    wp.y = w.ymin;
                } else {
                    wp.x = w.xmin;
                    wp.y = wval;
                }
                
                if (wval < wmin || wval > wmax) {
                    continue;
                }
                
                this.wPoint2vPoint(wp, vp1);
                // alert(wp.x.toString() + " " + wp.y.toString());
                
                var major_tick;
                var tick_length;
                if (i % 2) {
                    major_tick = false;
                    tick_length = 0.01;
                } else {
                    major_tick = true;
                    tick_length = 0.02;
                }
                
                if (axis == "x") {
                    vp2.x = vp1.x;
                    vp2.y = vp1.y + tick_length;
                } else {
                    vp2.x = vp1.x + tick_length;
                    vp2.y = vp1.y;
                }
                this.drawLineV(vp1, vp2);
                
                // Tick label offset
                if (axis == "x") {
                    vp1.y -= 0.01;
                } else {
                    vp1.x -= 0.01;
                }
                
                if (major_tick) {
                    this.drawStringV(vp1,
                        pf.util.Format.numPrint(wval, prec), halign, valign);
                }
            }
        },
        
        drawTitle: function(title, color)
        {
            var v = this.v;

            this.setColor(color);
            
            var offset = 0.03;
            
            var vp = aPoint((v.xmin + v.xmax)/2, v.ymax + offset);
            
            this.drawStringV(vp, title, "center", "bottom");
        }
    }
});
